Attribute VB_Name = "ExtractArgs"
' ExtractArgs module
' Ver 1.0   21/4/01
' This module will take a line and split it into various arguments or sections
'
' usage: ArgExtract(Separator As String, Comment As String, QuotesOK As Boolean, Line As String)
' where:    Separator = character or string that is separating the arguments
'           Comment = character or string that represents a comment.  Anything beyond the comment
'                     is ignored.  If no comments are required set to a null string (eg. "")
'           QuotesOK = If true will search for quotes around arguments.  The Separator and Comment
'                      characters can be within the quotes and will be ignored in that case.
'                      Note: Characters before and after the quotes will be ignored, for example
'                      the string df"gggg"se will return gggg as the argument.
'           Line = is the string to be searched.
' The results are returned four ways:
'       The ArgExtract function will return false if an error was detected
'       The public variable ArgError will hold a string describing the error
'       The public variable ArgC will hold the number of arguments found
'       The public array ArgV(0 to 99) will hold the various arguments
'
'
Option Explicit

Public Const ArgMax = 99
Public ArgV(0 To ArgMax) As String
Public ArgC As Integer
Public ArgError As String

Public Function ArgExtract(Separator As String, Comment As String, QuotesOK As Boolean, Line As String) As Boolean
    Dim LineLen As Integer
    Dim e As Integer
    Dim t As Integer
    Dim p As Integer
    Dim skip As Integer
    
    ' The separator must be specified
    If Separator = "" Then
        Err.Raise vbObjectError + 1, "ArgExtract", "Separator of zero length in ArgExtract()"
        Exit Function
    End If
    
    ' Determine the useable length of the line, eg.  without the comment character
    If Comment <> "" And InStr(Line, Comment) > 0 Then
        LineLen = InStr(Line, Comment) - 1
    Else
        LineLen = Len(Line)
    End If
    
    ArgC = 0
    If Len(Line) = 0 Then
        ArgExtract = True
        Exit Function
    End If
    
    p = 1
    While p <= LineLen
        e = InStr(p, Line, Separator)                   ' find the next separator
        If e <= 0 Or e > LineLen Then e = LineLen + 1   ' and correct if not there or beyond the comment char
        If QuotesOK Then                                ' this block allows quote marks
            t = InStr(p, Line, """")                    ' find the next quote
            If t > 0 And t < e Then                     ' if the quote exists and less than the next separator
                p = t + 1                               ' set the start pointer to just beyond the first quote
                e = p
LoopOverPairsOfQuotes:
                e = InStr(e, Line, """")            ' find the second quote
                If e <= 0 Then                          ' error if no second quote (must be balanced)
                    ArgError = "Quote mark ("") withour matching closing quote mark"
                    Exit Function
                End If
                If e < LineLen And Mid$(Line, e + 1, 1) = """" Then     ' check if this is a pair of double quotes
                    e = e + 2                           ' step over them if it is
                    GoTo LoopOverPairsOfQuotes          ' and look some more
                End If
                ArgV(ArgC) = Mid$(Line, p, e - p)       ' extract the arg from between the quotes
                If Comment <> "" And e >= LineLen Then
                    LineLen = InStr(e, Line, Comment) - 1 ' correct if there was a comment char between the quotes
                    If LineLen <= 0 Then LineLen = Len(Line)
                End If
                e = InStr(e, Line, Separator)           ' find the separator after the closing quote
                If e <= 0 Then e = LineLen              ' and correct if non existent
            Else                                        ' else no quotes found
                ArgV(ArgC) = Mid$(Line, p, e - p)       ' extract the arg
            End If
        Else                                            ' else if not looking for quotes
            ArgV(ArgC) = Mid$(Line, p, e - p)           ' extract the arg
        End If
        ArgC = ArgC + 1                                 ' increment the argument count
        p = e + Len(Separator)                          ' point to the start of the next arg
    Wend
    
    ArgExtract = True
    
End Function
